#!/bin/bash
# Depends: sqlite3
set -euo pipefail
mailing_list_name=$1
form_dir=$2
excluded=$3

# Path to the Evolution contacts database
db_file=~/.local/share/evolution/addressbook/system/contacts.db

# Backup the database
cp "$db_file" "$db_file.backup"

# Define an array of contacts (name and email pairs)
#
# Reads files in $form_dir with structure like
#
#  Name: Som E. Name
#  Email: address@example.com
mapfile -t contacts < <(find ${form_dir} -type f -name "*.txt" -exec awk -F ":" '
/Name:/ { name = $2 } /Email:/ { email = $2 } END {
  printf "%s <%s>\n", substr(name,2), substr(email,2)
}
' {} \; | grep -Fx -f "$excluded" -v)


# Loop through the contacts array and add each contact to the mailing list
for contact_email in "${contacts[@]}"; do

    # Use sqlite3 to add the contact to the mailing list
    #set -x
    exists=$(sqlite3 "$db_file" <<EOF
.parameter init
.parameter set @email "'$contact_email'"
SELECT EXISTS (SELECT * FROM folder_id_email_list WHERE value = @email);
EOF
)
    [ "$exists" == "1" ] && echo "Skipped (exists): '$contact_email'" && continue
    vcard=$(sqlite3 "$db_file" <<EOF
.parameter init
.parameter set @list "'${mailing_list_name,,}'"
SELECT vcard FROM folder_id WHERE file_as = @list;
EOF
)

    vcard=$(awk \
      -v new_email="EMAIL;X-EVOLUTION-DEST-HTML-MAIL=FALSE;X-EVOLUTION-PARENT-UID=0:$contact_email" \
      -v max_width=75 '
BEGIN { FS=":"; OFS=":" }
/BEGIN:VCARD/ { print $0; p=1; next }
/END:VCARD/ {
  while (length(new_email) > max_width) {
    print substr(new_email, 1, max_width)
    new_email = " " substr(new_email, max_width + 1)
  }
  print new_email
  print $0
  next
}
p
' <<< "$vcard")
    #read -p "$vcard"
    vcard=$(awk '{printf "%s\\n", $0}' <<< "$vcard")

    sqlite3 "$db_file" <<EOF &&  echo "Added: '$contact_email'"
.parameter init
.parameter set @list "'${mailing_list_name,,}'"
.parameter set @vcard "'${vcard}'"
.parameter set @email "'$contact_email'"
UPDATE folder_id SET vcard = @vcard WHERE file_as = @list;
INSERT INTO folder_id_email_list (uid, value)
  VALUES ((SELECT uid FROM folder_id WHERE file_as = @list), @email);
EOF
done

echo "Restart evolution:"
echo "  - If open, first do:  File -> Quit"
echo "  - Then once closed do (in terminal):  evolution --force-shutdown"

